@@ -1,6 +1,6 @@
 package Courriel;
 {
-  $Courriel::VERSION = '0.19';
+  $Courriel::VERSION = '0.23';
 }
 
 use 5.10.0;
@@ -9,9 +9,8 @@ use strict;
 use warnings;
 use namespace::autoclean;
 
-use Courriel::ContentType;
 use Courriel::Headers;
-use Courriel::Helpers qw( parse_header_with_attributes unique_boundary );
+use Courriel::Helpers qw( unique_boundary );
 use Courriel::Part::Multipart;
 use Courriel::Part::Single;
 use Courriel::Types
@@ -20,6 +19,7 @@ use DateTime;
 use DateTime::Format::Mail;
 use DateTime::Format::Natural;
 use Email::Address;
+use Encode qw( encode );
 use List::AllUtils qw( uniq );
 use MooseX::Params::Validate qw( validated_list );
 
@@ -151,7 +151,7 @@ sub clone_without_attachments {
     my $headers = $self->headers();
 
     if ( $plain_body && $html_body ) {
-        my $ct = Courriel::ContentType->new(
+        my $ct = Courriel::Header::ContentType->new(
             mime_type  => 'multipart/alternative',
             attributes => { boundary => unique_boundary() },
         );
@@ -191,7 +191,9 @@ sub clone_without_attachments {
 sub _build_subject {
     my $self = shift;
 
-    return $self->headers()->get('Subject');
+    my $subject = $self->headers()->get('Subject');
+
+    return $subject ? $subject->value() : undef;
 }
 
 {
@@ -202,13 +204,13 @@ sub _build_subject {
         my $self = shift;
 
         my @possible = (
-            $self->headers()->get('Date'),
+            ( map { $_->value() } $self->headers()->get('Date') ),
             (
                 reverse
-                    map { $self->_find_date_received($_) }
+                    map { $self->_find_date_received( $_->value() ) }
                     $self->headers()->get('Received')
             ),
-            $self->headers()->get('Resent-Date')
+            ( map { $_->value() } $self->headers()->get('Resent-Date') ),
         );
 
         # Stolen from Email::Date and then modified
@@ -245,8 +247,8 @@ sub _find_date_received {
 sub _build_to {
     my $self = shift;
 
-    my @addresses
-        = map { Email::Address->parse($_) } $self->headers()->get('To');
+    my @addresses = map { Email::Address->parse( $_->value() ) }
+        $self->headers()->get('To');
 
     return $self->_unique_addresses( \@addresses );
 }
@@ -254,8 +256,8 @@ sub _build_to {
 sub _build_cc {
     my $self = shift;
 
-    my @addresses
-        = map { Email::Address->parse($_) } $self->headers()->get('CC');
+    my @addresses = map { Email::Address->parse( $_->value() ) }
+        $self->headers()->get('CC');
 
     return $self->_unique_addresses( \@addresses );
 }
@@ -263,7 +265,8 @@ sub _build_cc {
 sub _build_from {
     my $self = shift;
 
-    my @addresses = Email::Address->parse( $self->headers()->get('From') );
+    my @addresses = Email::Address->parse( map { $_->value() }
+            $self->headers()->get('From') );
 
     return $addresses[0];
 }
@@ -370,22 +373,29 @@ sub all_parts_matching {
 }
 
 {
-    my @spec = ( text => { isa => StringRef, coerce => 1 } );
+    my @spec = (
+        text      => { isa => StringRef, coerce  => 1 },
+        is_binary => { isa => Bool,      default => 1 },
+    );
 
     sub parse {
         my $class = shift;
-        my ($text) = validated_list(
+        my ( $text, $is_binary ) = validated_list(
             \@_,
             @spec,
         );
 
+        if ( !$is_binary ) {
+            ${$text} = encode( 'utf-8', ${$text} );
+        }
+
         return $class->new( part => $class->_parse($text) );
     }
 }
 
 sub _parse {
-    my $class = shift;
-    my $text  = shift;
+    my $class     = shift;
+    my $text      = shift;
 
     my ( $line_sep, $sep_idx, $headers ) = $class->_parse_headers($text);
 
@@ -428,24 +438,49 @@ sub _parse_headers {
     return ( $line_sep, $sep_idx, $headers );
 }
 
-sub _parse_parts {
+{
+    my $fake_ct = Courriel::Header::ContentType->new_from_value(
+        name  => 'Content-Type',
+        value => 'text/plain'
+    );
+
+    sub _parse_parts {
+        my $class   = shift;
+        my $text    = shift;
+        my $headers = shift;
+
+        my @ct = $headers->get('Content-Type');
+        if ( @ct > 1 ) {
+            die 'This email defines more than one Content-Type header.';
+        }
+
+        my $ct = $ct[0] // $fake_ct;
+
+        if ( $ct->mime_type() !~ /^multipart/i ) {
+            return Courriel::Part::Single->new(
+                headers         => $headers,
+                encoded_content => $text,
+            );
+        }
+
+        return $class->_parse_multipart( $text, $headers, $ct );
+    }
+}
+
+sub _parse_multipart {
     my $class   = shift;
     my $text    = shift;
     my $headers = shift;
+    my $ct      = shift;
 
-    my ( $mime, $ct_attr ) = $class->_content_type_from_headers($headers);
+    my $boundary_attr = $ct->attribute('boundary');
 
-    if ( $mime !~ /^multipart/ ) {
-        return Courriel::Part::Single->new(
-            headers         => $headers,
-            encoded_content => $text,
-        );
-    }
+    die q{The message's mime type claims this is a multipart message (}
+        . $ct->mime_type()
+        . q{) but it does not specify a boundary.}
+        unless $boundary_attr && length $boundary_attr->value();
 
-    my $boundary = $ct_attr->{boundary}
-        // die q{The message's mime type claims this is a multipart message (}
-        . $mime
-        . q{) but it does not specify a boundary.};
+    my $boundary = $boundary_attr->value();
 
     my ( $preamble, $all_parts, $epilogue ) = ${$text} =~ /
                 (.*?)                   # preamble
@@ -479,24 +514,10 @@ sub _parse_parts {
                 && $epilogue =~ /\S/ ? ( epilogue => $epilogue ) : ()
         ),
         boundary => $boundary,
-        parts => [ map { $class->_parse( \$_ ) } @part_text ],
+        parts    => [ map { $class->_parse( \$_ ) } @part_text ],
     );
 }
 
-sub _content_type_from_headers {
-    my $class   = shift;
-    my $headers = shift;
-
-    my @ct = $headers->get('Content-Type');
-    if ( @ct > 1 ) {
-        die 'This email defines more than one Content-Type header.';
-    }
-
-    return defined $ct[0]
-        ? parse_header_with_attributes( $ct[0] )
-        : ( 'text/plain', {} );
-}
-
 __PACKAGE__->meta()->make_immutable();
 
 1;
@@ -513,7 +534,7 @@ Courriel - High level email parsing and manipulation
 
 =head1 VERSION
 
-version 0.19
+version 0.23
 
 =head1 SYNOPSIS
 
@@ -545,7 +566,7 @@ classes.
 
 This class provides the following methods:
 
-=head2 Courriel->parse( text => $raw_email )
+=head2 Courriel->parse( text => $raw_email, is_binary => 0|1 )
 
 This parses the given text and returns a new Courriel object. The text can be
 provided as a string or a reference to a string.
@@ -553,6 +574,21 @@ provided as a string or a reference to a string.
 If you pass a reference, then the scalar underlying the reference I<will> be
 modified, so don't pass in something you don't want modified.
 
+By default, Courriel expects that content passed in text is binary data. This
+means that it has not been decoded into utf-8 with L<Encode::decode()> or by
+using a C<:utf8> IO layer.
+
+In practice, this doesn't matter for most emails, since they either contain
+only ASCII data or they actually do contain binary (non-character)
+data. However, if an email is using the 8bit Content-Transfer-Encoding, then
+this does matter.
+
+If you the email has already been decoded, you must set C<is_binary> to a
+false value.
+
+It's probably safest to simply pass binary data to Courriel and let it handle
+decoding internally.
+
 =head2 $email->parts()
 
 Returns an array (not a reference) of the parts this email contains.
@@ -654,7 +690,7 @@ This method returns all of the parts that match the subroutine.
 
 =head2 $email->content_type()
 
-Returns the L<Courriel::ContentType> object associated with the email.
+Returns the L<Courriel::Header::ContentType> object associated with the email.
 
 =head2 $email->headers()
 
@@ -665,6 +701,21 @@ Returns the L<Courriel::Headers> object for this email.
 Returns the email as a string, along with its headers. Lines will be
 terminated with "\r\n".
 
+=head1 ROBUSTNESS PRINCIPLE
+
+Courriel aims to respect the common Internet robustness principle (aka
+Postel's law). Courriel is conservative in the output it generates, and
+liberal in what it accepts.
+
+When parsing, the goal is to never die and always return as much information
+as possible. Any input that causes the C<< Courriel->parse() >> to die means
+there's a bug in the parser. Please report these bugs.
+
+Conversely, Courriel aims to respect all relevant RFCs in its output, except
+when it preserves the original data in a parsed email. If you're using
+L<Courriel::Builder> to create emails from scratch, any output that isn't
+RFC-compliant is a bug.
+
 =head1 FUTURE PLANS
 
 This release is still rough, and I have some plans for additional features:
