@@ -1,19 +1,23 @@
 package Courriel::Headers;
 {
-  $Courriel::Headers::VERSION = '0.19';
+  $Courriel::Headers::VERSION = '0.23';
 }
 
 use strict;
 use warnings;
 use namespace::autoclean;
 
+use Courriel::Header;
+use Courriel::Header::ContentType;
+use Courriel::Header::Disposition;
 use Courriel::Helpers qw( fold_header );
 use Courriel::Types
-    qw( ArrayRef Defined EvenArrayRef HashRef NonEmptyStr Str StringRef );
+    qw( ArrayRef Defined HashRef HeaderArray NonEmptyStr Str StringRef );
 use Encode qw( decode encode find_encoding );
 use MIME::Base64 qw( decode_base64 encode_base64 );
 use MIME::QuotedPrint qw( decode_qp );
 use MooseX::Params::Validate qw( pos_validated_list validated_list );
+use Scalar::Util qw( blessed reftype );
 
 use Moose;
 use MooseX::StrictConstructor;
@@ -21,7 +25,7 @@ use MooseX::StrictConstructor;
 has _headers => (
     traits   => ['Array'],
     is       => 'ro',
-    isa      => EvenArrayRef [Str],
+    isa      => HeaderArray,
     default  => sub { [] },
     init_arg => 'headers',
     handles  => {
@@ -54,6 +58,59 @@ has _key_indices => (
     },
 );
 
+override BUILDARGS => sub {
+    my $class = shift;
+
+    my $p = super();
+
+    return $p unless $p->{headers};
+
+    # Could this be done as a coercion for the HeaderArray type? Maybe, but
+    # it'd probably need structured types, which seems like as much of a
+    # hassle as just doing this.
+    if ( reftype( $p->{headers} ) eq 'ARRAY' ) {
+        my $headers = $p->{headers};
+
+        for ( my $i = 1 ; $i < @{$headers} ; $i += 2 ) {
+            next if blessed( $headers->[ $i - 1 ] );
+
+            my $name = $headers->[ $i - 1 ];
+
+            next unless defined $name;
+
+            $headers->[$i] = $class->_inflate_header( $name, $headers->[$i] );
+        }
+    }
+    elsif ( reftype( $p->{headers} ) eq 'HASH' ) {
+        for my $name ( keys %{ $p->{headers} } ) {
+            next if blessed( $p->{headers}{$name} );
+
+            $p->{headers}{$name}
+                = $class->_inflate_header( $name, $p->{headers}{$name} );
+        }
+    }
+
+    return $p;
+};
+
+sub _inflate_header {
+    my $class = shift;
+    my $name  = shift;
+    my $value = shift;
+
+    my ( $header_class, $method )
+        = lc $name eq 'content-type'
+        ? ( 'Courriel::Header::ContentType', 'new_from_value' )
+        : lc $name eq 'content-disposition'
+        ? ( 'Courriel::Header::Disposition', 'new_from_value' )
+        : ( 'Courriel::Header', 'new' );
+
+    return $header_class->$method(
+        name  => $name,
+        value => $value,
+    );
+}
+
 sub _build_key_indices {
     my $self = shift;
 
@@ -72,20 +129,36 @@ sub _build_key_indices {
 
     sub get {
         my $self = shift;
-        my ($key) = pos_validated_list(
+        my ($name) = pos_validated_list(
             \@_,
             @spec,
         );
 
-        return @{ $self->_headers() }[ $self->_key_indices_for($key) ];
+        return @{ $self->_headers() }[ $self->_key_indices_for($name) ];
+    }
+}
+
+{
+    my @spec = ( { isa => NonEmptyStr } );
+
+    sub get_values {
+        my $self = shift;
+        my ($name) = pos_validated_list(
+            \@_,
+            @spec,
+        );
+
+        return
+            map { $_->value() }
+            @{ $self->_headers() }[ $self->_key_indices_for($name) ];
     }
 }
 
 sub _key_indices_for {
     my $self = shift;
-    my $key  = shift;
+    my $name = shift;
 
-    return @{ $self->__key_indices_for( lc $key ) || [] };
+    return @{ $self->__key_indices_for( lc $name ) || [] };
 }
 
 {
@@ -96,20 +169,26 @@ sub _key_indices_for {
 
     sub add {
         my $self = shift;
-        my ( $key, $val ) = pos_validated_list(
+        my ( $name, $value ) = pos_validated_list(
             \@_,
             @spec,
         );
 
         my $headers = $self->_headers();
 
-        my $last_index = ( $self->_key_indices_for($key) )[-1];
+        my $last_index = ( $self->_key_indices_for($name) )[-1];
+
+        my $header
+            = blessed($value)
+            && $value->isa('Courriel::Header')
+            ? $value
+            : $self->_inflate_header( $name, $value );
 
         if ($last_index) {
-            splice @{$headers}, $last_index + 1, 0, ( $key => $val );
+            splice @{$headers}, $last_index + 1, 0, ( $name => $header );
         }
         else {
-            push @{$headers}, ( $key => $val );
+            push @{$headers}, ( $name => $header );
         }
 
         $self->_clear_key_indices();
@@ -127,7 +206,7 @@ sub _key_indices_for {
     # Used to add things like Resent or Received headers
     sub unshift {
         my $self = shift;
-        my ( $key, $val ) = pos_validated_list(
+        my ( $name, $value ) = pos_validated_list(
             \@_,
             { isa => NonEmptyStr },
             ( { isa => Defined } ) x ( @_ - 1 ),
@@ -136,7 +215,13 @@ sub _key_indices_for {
 
         my $headers = $self->_headers();
 
-        unshift @{$headers}, ( $key => $val );
+        my $header
+            = blessed($value)
+            && $value->isa('Courriel::Header')
+            ? $value
+            : $self->_inflate_header( $name, $value );
+
+        unshift @{$headers}, ( $name => $header );
 
         return;
     }
@@ -149,14 +234,14 @@ sub _key_indices_for {
 
     sub remove {
         my $self = shift;
-        my ($key) = pos_validated_list(
+        my ($name) = pos_validated_list(
             \@_,
             @spec,
         );
 
         my $headers = $self->_headers();
 
-        for my $idx ( reverse $self->_key_indices_for($key) ) {
+        for my $idx ( reverse $self->_key_indices_for($name) ) {
             splice @{$headers}, $idx - 1, 2;
         }
 
@@ -174,13 +259,13 @@ sub _key_indices_for {
 
     sub replace {
         my $self = shift;
-        my ( $key, $val ) = pos_validated_list(
+        my ( $name, $value ) = pos_validated_list(
             \@_,
             @spec,
         );
 
-        $self->remove($key);
-        $self->add( $key => $val );
+        $self->remove($name);
+        $self->add( $name => $value );
 
         return;
     }
@@ -251,7 +336,7 @@ sub _key_indices_for {
                 : 'Could not parse headers at all';
         }
 
-        for ( my $i = 1; $i < @headers; $i += 2 ) {
+        for ( my $i = 1 ; $i < @headers ; $i += 2 ) {
             $headers[$i] = $class->_mime_decode( $headers[$i] );
         }
 
@@ -264,7 +349,7 @@ sub _maybe_fix_broken_headers {
     my $text   = shift;
     my $sep_re = shift;
 
-    # Some broken email messages have a newline int he headers that isn't
+    # Some broken email messages have a newline in the headers that isn't
     # acting as a continuation, it's just an arbitrary line break. See
     # t/data/stress-test/mbox_mime_applemail_1xb.txt
     ${$text} =~ s/$sep_re([^\s:][^:]+$sep_re)/$1/g;
@@ -294,7 +379,7 @@ sub _maybe_fix_broken_headers {
         for ( my $i = 0; $i < @{$headers}; $i += 2 ) {
             next if $skip{ lc $headers->[$i] };
 
-            my $value = $headers->[ $i + 1 ];
+            my $value = $headers->[ $i + 1 ]->value();
 
             $value = $self->_mime_encode( $value, $charset )
                 unless $value =~ /^[\x20-\x7e]+$/;
@@ -432,7 +517,7 @@ Courriel::Headers - The headers for an email part
 
 =head1 VERSION
 
-version 0.19
+version 0.23
 
 =head1 SYNOPSIS
 
@@ -449,6 +534,22 @@ Any sub part of an email can have its own headers, so every part has an
 associated object representing its headers. This class makes no distinction
 between top-level headers and headers for a sub part.
 
+Each individual header name/value pair is represented internally by a
+L<Courriel::Header> object. Some headers have their own special
+subclass. These are:
+
+=over 4
+
+=item * Content-Type
+
+This is stored as a L<Courriel::Header::ContentType> object.
+
+=item * Content-Disposition
+
+This is stored as a L<Courriel::Header::Disposition> object.
+
+=back
+
 =head1 API
 
 This class supports the following methods:
@@ -473,8 +574,9 @@ output with RFC-compliant line endings.
 
 =back
 
-Header parsing unfolds folded headers, and decodes any MIME-encoded values (as
-described in RFC 2047).
+Header parsing unfolds folded headers, and decodes any MIME-encoded values as
+described in RFC 2047. Parsing also decodes header attributes encoded as
+described in RFC 2231.
 
 =head2 Courriel::Headers->new( headers => [ ... ] )
 
@@ -490,8 +592,14 @@ Header order is preserved, per RFC 5322.
 
 =head2 $headers->get($name)
 
-Given a header name, this returns a list of the values found for the
-header. Each occurrence of the header is returned as a separate value.
+Given a header name, this returns a list of the L<Courriel::Header> objects
+found for the header. Each occurrence of the header is returned as a separate
+object.
+
+=head2 $headers->get_values($name)
+
+Given a header name, this returns a list of the string values found for the
+header. Each occurrence of the header is returned as a separate string.
 
 =head2 $headers->add( $name => $value )
 
@@ -499,12 +607,16 @@ Given a header name and value, this adds the headers to the object. If any of
 the headers already have values in the object, then new values are added after
 the existing values, rather than at the end of headers.
 
+The value can be provided as a string or a L<Courriel::Header> object.
+
 =head2 $headers->unshift( $name => $value )
 
 This is like C<add()>, but this pushes the headers onto the front of the
 internal headers array. This is useful if you are adding "Received" headers,
 which per RFC 5322, should always be added at the I<top> of the headers.
 
+The value can be provided as a string or a L<Courriel::Header> object.
+
 =head2 $headers->remove($name)
 
 Given a header name, this removes all instances of that header from the object.
@@ -513,6 +625,8 @@ Given a header name, this removes all instances of that header from the object.
 
 A shortcut for calling C<remove()> and C<add()>.
 
+The value can be provided as a string or a L<Courriel::Header> object.
+
 =head2 $headers->as_string( charset => ... )
 
 This returns a string representing the headers in the object. The values will
