@@ -1,15 +1,14 @@
 package Courriel::Role::Part;
 {
-  $Courriel::Role::Part::VERSION = '0.19';
+  $Courriel::Role::Part::VERSION = '0.23';
 }
 
 use strict;
 use warnings;
 use namespace::autoclean;
 
-use Courriel::ContentType;
-use Courriel::Disposition;
-use Courriel::Helpers qw( parse_header_with_attributes );
+use Courriel::Header::ContentType;
+use Courriel::Header::Disposition;
 
 use Courriel::Types qw( NonEmptyStr );
 
@@ -33,41 +32,13 @@ has container => (
 
 has content_type => (
     is        => 'ro',
-    isa       => 'Courriel::ContentType',
+    isa       => 'Courriel::Header::ContentType',
     lazy      => 1,
     builder   => '_build_content_type',
     predicate => '_has_content_type',
     handles   => [qw( mime_type charset has_charset )],
 );
 
-sub as_string {
-    my $self = shift;
-
-    return
-          $self->headers()->as_string()
-        . $Courriel::Helpers::CRLF
-        . $self->_content_as_string();
-}
-
-sub _build_content_type {
-    my $self = shift;
-
-    my @ct = $self->headers()->get('Content-Type');
-    if ( @ct > 1 ) {
-        die 'This part defines more than one Content-Type header.';
-    }
-
-    my ( $mime, $attr )
-        = defined $ct[0]
-        ? parse_header_with_attributes( $ct[0] )
-        : ( 'text/plain', {} );
-
-    return Courriel::ContentType->new(
-        mime_type  => $mime,
-        attributes => $attr,
-    );
-}
-
 after BUILD => sub {
     my $self = shift;
 
@@ -89,9 +60,34 @@ sub _maybe_set_content_type_in_headers {
 
     return unless $self->_has_content_type();
 
-    $self->headers()
-        ->replace(
-        'Content-Type' => $self->content_type()->as_header_value() );
+    $self->headers()->replace( 'Content-Type' => $self->content_type() );
+}
+
+sub as_string {
+    my $self = shift;
+
+    return
+          $self->headers()->as_string()
+        . $Courriel::Helpers::CRLF
+        . $self->_content_as_string();
+}
+
+{
+    my $fake_ct = Courriel::Header::ContentType->new_from_value(
+        name  => 'Content-Type',
+        value => 'text/plain'
+    );
+
+    sub _build_content_type {
+        my $self = shift;
+
+        my @ct = $self->headers()->get('Content-Type');
+        if ( @ct > 1 ) {
+            die 'This part defines more than one Content-Type header.';
+        }
+
+        return $ct[0] // $fake_ct;
+    }
 }
 
 1;
